@@ -11,8 +11,10 @@
 /* @(#)pinyin-private.h
  */
 
+/* *INDENT-OFF* */
 #ifndef _CHEWING_PINYIN_PRIVATE_H
 #define _CHEWING_PINYIN_PRIVATE_H
+/* *INDENT-ON* */
 
 #include "chewing-private.h"
 
@@ -21,14 +23,15 @@
   From pinyin sequence to a default-layout sequence.
   Eg: Zhang -> {"zh","5"}, {"ang",";"}
  */
-struct keymap {
-	char pinyin[7];
-	char zuin[4];
-};
-typedef struct keymap keymap;
+typedef struct keymap {
+    char pinyin[7];
+    char bopomofo[4];
+} keymap;
 
-int PinyinToZuin( ChewingData *pgdata, const char *pinyinKeySeq, char *zuinKeySeq, char *zuinKeySeqAlt);
-int InitPinyin( ChewingData *pgdata, const char * );
-void TerminatePinyin( ChewingData *pgdata );
+int PinyinToBopomofo(ChewingData *pgdata, const char *pinyinKeySeq, char *bopomofoKeySeq, char *bopomofoKeySeqAlt);
+int InitPinyin(ChewingData *pgdata, const char *);
+void TerminatePinyin(ChewingData *pgdata);
 
+/* *INDENT-OFF* */
 #endif
+/* *INDENT-ON* */
