@@ -32,10 +32,10 @@
 /*
  * process a key input
  * return value:
- *	BOPOMOFO_ABSORB
- *	BOPOMOFO_COMMIT
- *	BOPOMOFO_KEY_ERROR
- *	BOPOMOFO_ERROR
+ *      BOPOMOFO_ABSORB
+ *      BOPOMOFO_COMMIT
+ *      BOPOMOFO_KEY_ERROR
+ *      BOPOMOFO_ERROR
  */
 static int IsHsuPhoEndKey(const int pho_inx[], int key)
 {
@@ -216,6 +216,7 @@ static int HsuPhoInput(ChewingData *pgdata, int key)
             }
         }
 
+        /* fuzzy ㄍㄧ to ㄐㄧ and ㄍㄩ to ㄐㄩ */
         if ((pBopomofo->pho_inx[0] == 9) && ((pBopomofo->pho_inx[1] == 1) || (pBopomofo->pho_inx[1] == 3))) {
             pBopomofo->pho_inx[0] = 12;
         }
@@ -230,7 +231,15 @@ static int HsuPhoInput(ChewingData *pgdata, int key)
             if (!inx)
                 continue;       /* if inx == 0, next type */
             else if (type == 0) {
-                if (pBopomofo->pho_inx[0] || pBopomofo->pho_inx[1]) {
+                /**
+                 * Hsu maps multiple bopomofo into one single key.
+                 * Therefore, if a consonant or a medial already exists
+                 * in buffer, and the user presses a key with consonant
+                 * and rhyme, libchewing should consider that the user
+                 * wants to input the rhyme.
+                 */
+                if ((inx == 3 || (7 <= inx && inx <= 11) || inx == 20)
+                    && (pBopomofo->pho_inx[0] || pBopomofo->pho_inx[1])) {
                     /* if inx !=0 */
                     searchTimes = 2;    /* possible infinite loop here */
                 } else
@@ -243,18 +252,20 @@ static int HsuPhoInput(ChewingData *pgdata, int key)
             } else
                 break;
         }
-        /* processing very special cases "j v c" */
-        if (type == 1 && inx == 2 && 12 <= pBopomofo->pho_inx[0] && pBopomofo->pho_inx[0] <= 14) {
-            pBopomofo->pho_inx[0] += 3;
-        }
 
-        /* Fuzzy "g e" to "j e" */
+        /* fuzzy ㄍㄧ to ㄐㄧ and ㄍㄩ to ㄐㄩ */
         if ((pBopomofo->pho_inx[0] == 9) && ((pBopomofo->pho_inx[1] == 1) || (pBopomofo->pho_inx[1] == 3))) {
             pBopomofo->pho_inx[0] = 12;
         }
 
-        /* ㄐㄑㄒ must follow ㄧㄩ */
+        /* ㄐㄑㄒ must be followed by ㄧㄩ, if not, convert them to ㄓㄔㄕ */
+        if (type == 1 && inx == 2 && 12 <= pBopomofo->pho_inx[0] && pBopomofo->pho_inx[0] <= 14) {
+            /* followed by ㄨ */
+            pBopomofo->pho_inx[0] += 3;
+        }
+
         if (type == 2 && pBopomofo->pho_inx[1] == 0 && 12 <= pBopomofo->pho_inx[0] && pBopomofo->pho_inx[0] <= 14) {
+            /* followed by other phones */
             pBopomofo->pho_inx[0] += 3;
         }
 
@@ -321,7 +332,14 @@ static int ET26PhoInput(ChewingData *pgdata, int key)
             if (!inx)
                 continue;       /* if inx == 0, next type */
             else if (type == 0) {
-                if (pBopomofo->pho_inx[0] || pBopomofo->pho_inx[1]) {
+                 /**
+                  * Same as Hsu: If a consonant or a medial already exists
+                  * in buffer, and the user presses a key with consonant
+                  * and rhyme, libchewing should consider that the user
+                  * wants to input the rhyme.
+                  */
+                if ((inx == 2 || inx == 3 || inx == 11 || inx == 19 || inx == 20 ||
+                    (6 <= inx && inx <= 8)) && (pBopomofo->pho_inx[0] || pBopomofo->pho_inx[1])) {
                     /* if inx !=0 */
                     searchTimes = 2;    /* possible infinite loop here */
                 } else
@@ -419,7 +437,7 @@ static int DACHENCP26PhoInput(ChewingData *pgdata, int key)
                 return BOPOMOFO_ABSORB;
             }
         }
-        /* converting "ㄙ" to "ㄣ" */
+        /* converting "ㄙ" to "ㄥ" */
         else if (key == 'n') {
             if (pBopomofo->pho_inx[0] != 0 || pBopomofo->pho_inx[1] != 0) {
                 pBopomofo->pho_inx[2] = 12;
