@@ -1,8 +1,8 @@
 /**
- * test-keyboardless-api.c
+ * test-keyboardless.c
  *
  * Copyright (c) 2013
- *	libchewing Core Team. See ChangeLog for details.
+ *      libchewing Core Team. See ChangeLog for details.
  *
  * See the file "COPYING" for information on usage and redistribution
  * of this file.
